@@ -1,6 +1,6 @@
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 // Mobius Forensic Toolkit
-// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020 Eduardo Aguiar
+// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022 Eduardo Aguiar
 //
 // This program is free software; you can redistribute it and/or modify it
 // under the terms of the GNU General Public License as published by the
@@ -31,7 +31,7 @@
 //! \return new data_decoder object
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 PyObject *
-PyMobius_Decoder_Data_Decoder_from_cpp (mobius::decoder::data_decoder obj)
+pymobius_decoder_data_decoder_to_pyobject (mobius::decoder::data_decoder obj)
 {
   PyObject *ret = _PyObject_New (&decoder_data_decoder_t);
 
@@ -49,11 +49,18 @@ PyMobius_Decoder_Data_Decoder_from_cpp (mobius::decoder::data_decoder obj)
 static PyObject *
 tp_f_skip (decoder_data_decoder_o *self, PyObject *args)
 {
-  // parse input args
+  // Parse input args
   std::uint64_t arg_size;
 
-  if (!PyArg_ParseTuple (args, "K", &arg_size))
-    return nullptr;
+  try
+    {
+      arg_size = mobius::py::get_arg_as_uint64_t (args, 0);
+    }
+  catch (const std::exception& e)
+    {
+      mobius::py::set_invalid_type_error (e.what ());
+      return nullptr;
+    }
 
   // execute C++ function
   PyObject *ret = nullptr;
@@ -61,13 +68,11 @@ tp_f_skip (decoder_data_decoder_o *self, PyObject *args)
   try
     {
       self->obj->skip (arg_size);
-
-      Py_INCREF (Py_None);
-      ret = Py_None;
+      ret = mobius::py::pynone ();
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -82,25 +87,30 @@ tp_f_skip (decoder_data_decoder_o *self, PyObject *args)
 static PyObject *
 tp_f_seek (decoder_data_decoder_o *self, PyObject *args)
 {
-  // parse input args
+  // Parse input args
   std::uint64_t arg_size;
 
-  if (!PyArg_ParseTuple (args, "K", &arg_size))
-    return nullptr;
-
+  try
+    {
+      arg_size = mobius::py::get_arg_as_uint64_t (args, 0);
+    }
+  catch (const std::exception& e)
+    {
+      mobius::py::set_invalid_type_error (e.what ());
+      return nullptr;
+    }
+    
   // execute C++ function
   PyObject *ret = nullptr;
 
   try
     {
       self->obj->seek (arg_size);
-
-      Py_INCREF (Py_None);
-      ret = Py_None;
+      ret = mobius::py::pynone ();
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -121,11 +131,11 @@ tp_f_tell (decoder_data_decoder_o *self, PyObject *)
 
   try
     {
-      ret = PyLong_FromUnsignedLongLong (self->obj->tell ());
+      ret = mobius::py::pylong_from_std_uint64_t (self->obj->tell ());
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -146,11 +156,11 @@ tp_f_get_size (decoder_data_decoder_o *self, PyObject *)
 
   try
     {
-      ret = PyLong_FromUnsignedLongLong (self->obj->get_size ());
+      ret = mobius::py::pylong_from_std_uint64_t (self->obj->get_size ());
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -171,11 +181,11 @@ tp_f_get_int8 (decoder_data_decoder_o *self, PyObject *)
 
   try
     {
-      ret = PyLong_FromLong (self->obj->get_int8 ());
+      ret = mobius::py::pylong_from_std_int8_t (self->obj->get_int8 ());
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -196,11 +206,11 @@ tp_f_get_int16_le (decoder_data_decoder_o *self, PyObject *)
 
   try
     {
-      ret = PyLong_FromLong (self->obj->get_int16_le ());
+      ret = mobius::py::pylong_from_std_int16_t (self->obj->get_int16_le ());
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -221,11 +231,11 @@ tp_f_get_int32_le (decoder_data_decoder_o *self, PyObject *)
 
   try
     {
-      ret = PyLong_FromLong (self->obj->get_int32_le ());
+      ret = mobius::py::pylong_from_std_int32_t (self->obj->get_int32_le ());
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -246,11 +256,11 @@ tp_f_get_uint8 (decoder_data_decoder_o *self, PyObject *)
 
   try
     {
-      ret = PyLong_FromLong (self->obj->get_uint8 ());
+      ret = mobius::py::pylong_from_std_uint8_t (self->obj->get_uint8 ());
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -271,11 +281,11 @@ tp_f_get_uint16_le (decoder_data_decoder_o *self, PyObject *)
 
   try
     {
-      ret = PyLong_FromLong (self->obj->get_uint16_le ());
+      ret = mobius::py::pylong_from_std_uint16_t (self->obj->get_uint16_le ());
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -296,11 +306,11 @@ tp_f_get_uint32_le (decoder_data_decoder_o *self, PyObject *)
 
   try
     {
-      ret = PyLong_FromLong (self->obj->get_uint32_le ());
+      ret = mobius::py::pylong_from_std_uint32_t (self->obj->get_uint32_le ());
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -321,11 +331,11 @@ tp_f_get_uint64_le (decoder_data_decoder_o *self, PyObject *)
 
   try
     {
-      ret = PyLong_FromUnsignedLongLong (self->obj->get_uint64_le ());
+      ret = mobius::py::pylong_from_std_uint64_t (self->obj->get_uint64_le ());
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -346,11 +356,11 @@ tp_f_get_uint16_be (decoder_data_decoder_o *self, PyObject *)
 
   try
     {
-      ret = PyLong_FromLong (self->obj->get_uint16_be ());
+      ret = mobius::py::pylong_from_std_uint16_t (self->obj->get_uint16_be ());
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -371,11 +381,11 @@ tp_f_get_uint32_be (decoder_data_decoder_o *self, PyObject *)
 
   try
     {
-      ret = PyLong_FromLong (self->obj->get_uint32_be ());
+      ret = mobius::py::pylong_from_std_uint32_t (self->obj->get_uint32_be ());
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -396,11 +406,11 @@ tp_f_get_uint64_be (decoder_data_decoder_o *self, PyObject *)
 
   try
     {
-      ret = PyLong_FromUnsignedLongLong (self->obj->get_uint64_be ());
+      ret = mobius::py::pylong_from_std_uint64_t (self->obj->get_uint64_be ());
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -425,7 +435,7 @@ tp_f_get_hfs_datetime (decoder_data_decoder_o *self, PyObject *)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -450,7 +460,7 @@ tp_f_get_iso9660_datetime (decoder_data_decoder_o *self, PyObject *)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -475,7 +485,7 @@ tp_f_get_nt_datetime (decoder_data_decoder_o *self, PyObject *)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -500,7 +510,7 @@ tp_f_get_unix_datetime (decoder_data_decoder_o *self, PyObject *)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -525,7 +535,7 @@ tp_f_get_fat_datetime (decoder_data_decoder_o *self, PyObject *)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -541,11 +551,18 @@ tp_f_get_fat_datetime (decoder_data_decoder_o *self, PyObject *)
 static PyObject *
 tp_f_get_bytearray_by_size (decoder_data_decoder_o *self, PyObject *args)
 {
-  // parse input args
+  // Parse input args
   std::uint64_t arg_size;
 
-  if (!PyArg_ParseTuple (args, "K", &arg_size))
-    return nullptr;
+  try
+    {
+      arg_size = mobius::py::get_arg_as_uint64_t (args, 0);
+    }
+  catch (const std::exception& e)
+    {
+      mobius::py::set_invalid_type_error (e.what ());
+      return nullptr;
+    }
 
   // execute C++ function
   PyObject *ret = nullptr;
@@ -556,7 +573,7 @@ tp_f_get_bytearray_by_size (decoder_data_decoder_o *self, PyObject *args)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -572,23 +589,31 @@ tp_f_get_bytearray_by_size (decoder_data_decoder_o *self, PyObject *args)
 static PyObject *
 tp_f_get_string_by_size (decoder_data_decoder_o *self, PyObject *args)
 {
-  // parse input args
+  // Parse input args
   std::uint64_t arg_size;
-  const char *arg_encode = "ASCII";
+  std::string arg_encode;
 
-  if (!PyArg_ParseTuple (args, "K|s", &arg_size, &arg_encode))
-    return nullptr;
+  try
+    {
+      arg_size = mobius::py::get_arg_as_uint64_t (args, 0);
+      arg_encode = mobius::py::get_arg_as_std_string (args, 1, "ASCII");
+    }
+  catch (const std::exception& e)
+    {
+      mobius::py::set_invalid_type_error (e.what ());
+      return nullptr;
+    }
 
   // execute C++ function
   PyObject *ret = nullptr;
 
   try
     {
-      ret = PyString2_from_std_string (self->obj->get_string_by_size (arg_size, arg_encode));
+      ret = mobius::py::pystring_from_std_string (self->obj->get_string_by_size (arg_size, arg_encode));
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -604,22 +629,29 @@ tp_f_get_string_by_size (decoder_data_decoder_o *self, PyObject *args)
 static PyObject *
 tp_f_get_c_string (decoder_data_decoder_o *self, PyObject *args)
 {
-  // parse input args
-  const char *arg_encode = "ASCII";
+  // Parse input args
+  std::string arg_encode;
 
-  if (!PyArg_ParseTuple (args, "|s", &arg_encode))
-    return nullptr;
+  try
+    {
+      arg_encode = mobius::py::get_arg_as_std_string (args, 0, "ASCII");
+    }
+  catch (const std::exception& e)
+    {
+      mobius::py::set_invalid_type_error (e.what ());
+      return nullptr;
+    }
 
   // execute C++ function
   PyObject *ret = nullptr;
 
   try
     {
-      ret = PyString2_from_std_string (self->obj->get_c_string (arg_encode));
+      ret = mobius::py::pystring_from_std_string (self->obj->get_c_string (arg_encode));
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -635,22 +667,29 @@ tp_f_get_c_string (decoder_data_decoder_o *self, PyObject *args)
 static PyObject *
 tp_f_get_hex_string_by_size (decoder_data_decoder_o *self, PyObject *args)
 {
-  // parse input args
+  // Parse input args
   std::uint64_t arg_size;
 
-  if (!PyArg_ParseTuple (args, "K", &arg_size))
-    return nullptr;
+  try
+    {
+      arg_size = mobius::py::get_arg_as_uint64_t (args, 0);
+    }
+  catch (const std::exception& e)
+    {
+      mobius::py::set_invalid_type_error (e.what ());
+      return nullptr;
+    }
 
   // execute C++ function
   PyObject *ret = nullptr;
 
   try
     {
-      ret = PyString2_from_std_string (self->obj->get_hex_string_by_size (arg_size));
+      ret = mobius::py::pystring_from_std_string (self->obj->get_hex_string_by_size (arg_size));
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -671,11 +710,11 @@ tp_f_get_guid (decoder_data_decoder_o *self, PyObject *)
 
   try
     {
-      ret = PyString2_from_std_string (self->obj->get_guid ());
+      ret = mobius::py::pystring_from_std_string (self->obj->get_guid ());
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -696,11 +735,11 @@ tp_f_get_uuid (decoder_data_decoder_o *self, PyObject *)
 
   try
     {
-      ret = PyString2_from_std_string (self->obj->get_uuid ());
+      ret = mobius::py::pystring_from_std_string (self->obj->get_uuid ());
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -721,11 +760,11 @@ tp_f_get_sid (decoder_data_decoder_o *self, PyObject *)
 
   try
     {
-      ret = PyString2_from_std_string (self->obj->get_sid ());
+      ret = mobius::py::pystring_from_std_string (self->obj->get_sid ());
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -746,11 +785,11 @@ tp_f_get_ipv4 (decoder_data_decoder_o *self, PyObject *)
 
   try
     {
-      ret = PyString2_from_std_string (self->obj->get_ipv4 ());
+      ret = mobius::py::pystring_from_std_string (self->obj->get_ipv4 ());
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -771,11 +810,11 @@ tp_f_get_ipv4_be (decoder_data_decoder_o *self, PyObject *)
 
   try
     {
-      ret = PyString2_from_std_string (self->obj->get_ipv4_be ());
+      ret = mobius::py::pystring_from_std_string (self->obj->get_ipv4_be ());
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -796,11 +835,11 @@ tp_f_get_ipv4_le (decoder_data_decoder_o *self, PyObject *)
 
   try
     {
-      ret = PyString2_from_std_string (self->obj->get_ipv4_le ());
+      ret = mobius::py::pystring_from_std_string (self->obj->get_ipv4_le ());
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -999,36 +1038,37 @@ static PyMethodDef tp_methods[] =
 static PyObject *
 tp_new (PyTypeObject *type, PyObject *args, PyObject *)
 {
-  // parse input args
-  PyObject *arg_reader = nullptr;
-  const std::uint8_t *arg_data_buffer = nullptr;
-  Py_ssize_t arg_data_size = 0;
+  // Parse input args
+  mobius::io::reader arg_reader;
 
-  if (!PyArg_ParseTuple (args, "s#", &arg_data_buffer, &arg_data_size) &&
-      !PyArg_ParseTuple (args, "O!", &io_reader_t, &arg_reader))
-    return nullptr;
-
-  PyErr_Clear ();
-
-  // create object
-  decoder_data_decoder_o *self = (decoder_data_decoder_o *) type->tp_alloc (type, 0);
-
-  if (self)
+  try
     {
-      if (arg_data_buffer)
+      arg_reader = pymobius_io_reader_from_pyobject (mobius::py::get_arg (args, 0));
+    }
+  catch (const std::exception& e)
+    {
+      mobius::py::set_invalid_type_error (e.what ());
+      return nullptr;
+    }
+
+  // Create Python object
+  decoder_data_decoder_o *ret = reinterpret_cast <decoder_data_decoder_o *> (type->tp_alloc (type, 0));
+
+  if (ret)
+    {
+      try
         {
-          self->data = new mobius::bytearray (arg_data_buffer, arg_data_size);
-          self->obj = new mobius::decoder::data_decoder (* (self->data));
+          ret->obj = new mobius::decoder::data_decoder (arg_reader);
         }
-         
-      else
+      catch (const std::exception& e)
         {
-          self->obj = new mobius::decoder::data_decoder (PyMobius_IO_Reader_as_cpp (arg_reader));
-          self->data = nullptr;
+          Py_DECREF (ret);
+          mobius::py::set_runtime_error (e.what ());
+          ret = nullptr;
         }
     }
 
-  return (PyObject *) self;
+  return reinterpret_cast <PyObject *> (ret);
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -1039,7 +1079,6 @@ static void
 tp_dealloc (decoder_data_decoder_o *self)
 {
   delete self->obj;
-  delete self->data;
   self->ob_type->tp_free ((PyObject*) self);
 }
 
