@@ -1,6 +1,6 @@
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 // Mobius Forensic Toolkit
-// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021 Eduardo Aguiar
+// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022 Eduardo Aguiar
 //
 // This program is free software; you can redistribute it and/or modify it
 // under the terms of the GNU General Public License as published by the
@@ -30,31 +30,35 @@
 PyObject*
 func_new_disk_from_device_uid (PyObject *, PyObject *args)
 {
-  // parse arguments
-  const char *arg_uid = nullptr;
-
-  if (!PyArg_ParseTuple (args, "s", &arg_uid))
-    return nullptr;
+  // parse input args
+  std::string arg_uid;
 
+  try
+    {
+      arg_uid = mobius::py::get_arg_as_std_string (args, 0);
+    }
+  catch (const std::exception& e)
+    {
+      mobius::py::set_invalid_type_error (e.what ());
+      return nullptr;
+    }
   // execute C++ code
   PyObject *ret = nullptr;
 
   try
     {
-      auto disk = mobius::py::GIL () (
-		      mobius::disk::new_disk_from_device_uid (arg_uid)
-                  );
+      auto disk = mobius::disk::disk_device (
+             mobius::py::GIL () (
+                mobius::disk::new_disk_from_device_uid (arg_uid)
+             )
+      );
 
-      ret = _PyObject_New (&disk_disk_device_t);
-
-      if (ret)
-        ((disk_disk_device_o *) ret)->obj = new mobius::disk::disk_device (disk);
+      ret = pymobius_disk_disk_device_to_pyobject (disk);
     }
   catch (const std::runtime_error& e)
     {
-      PyErr_SetString (PyExc_IOError, e.what ());
+      mobius::py::set_io_error (e.what ());
     }
 
-  // return object
   return ret;
 }
